@@ -149,6 +149,17 @@ class IJobDefinition(metaclass=ABCMeta):
         This value can be None
         """
 
+    @abstractproperty
+    def provider(self):
+        """
+        The provider this job definition belongs to
+
+        .. note::
+            Technically this still can be None (a provider-less job may exist) but
+            it can only happen in testing. This mode is discouraged and will be eventually
+            forbidden. All job definition units must belong to a provider.
+        """
+
 
 class IJobResult(metaclass=ABCMeta):
     """
@@ -375,7 +386,7 @@ class IJobRunner(metaclass=ABCMeta):
     """
 
     @abstractmethod
-    def run_job(self, job):
+    def run_job(self, job, config=None, ui=None):
         """
         Run the specified job.
 
@@ -809,7 +820,7 @@ class IExecutionController(metaclass=ABCMeta):
     """
 
     @abstractmethod
-    def execute_job(self, job, config, extcmd_popen):
+    def execute_job(self, job, config, session_dir, extcmd_popen):
         """
         Execute the specified job using the specified subprocess-like object
 
@@ -821,6 +832,10 @@ class IExecutionController(metaclass=ABCMeta):
             provide values for missing environment variables that are required
             by the job (as expressed by the environ key in the job definition
             file).
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param extcmd_popen:
             A subprocess.Popen like object
         :returns:
