@@ -49,9 +49,9 @@ from plainbox.abc import ISessionStateController
 from plainbox.i18n import gettext as _
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyMissingError
-from plainbox.impl.job import JobDefinition
 from plainbox.impl.resource import ExpressionCannotEvaluateError
 from plainbox.impl.resource import ExpressionFailedError
+from plainbox.impl.resource import ResourceProgramError
 from plainbox.impl.resource import Resource
 from plainbox.impl.secure.config import Unset
 from plainbox.impl.secure.origin import JobOutputTextSource
@@ -59,6 +59,10 @@ from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
 from plainbox.impl.secure.rfc822 import gen_rfc822_records
 from plainbox.impl.session.jobs import JobReadinessInhibitor
+from plainbox.impl.signal import Signal
+from plainbox.impl.unit.job import JobDefinition
+from plainbox.impl.unit.template import TemplateUnit
+from plainbox.impl.validation import ValidationError
 from plainbox.vendor import extcmd
 
 __all__ = [
@@ -112,9 +116,15 @@ class CheckBoxSessionStateController(ISessionStateController):
         """
         direct = DependencyMissingError.DEP_TYPE_DIRECT
         resource = DependencyMissingError.DEP_TYPE_RESOURCE
-        return set(itertools.chain(
-            zip(itertools.repeat(direct), job.get_direct_dependencies()),
-            zip(itertools.repeat(resource), job.get_resource_dependencies())))
+        direct_deps = job.get_direct_dependencies()
+        try:
+            resource_deps = job.get_resource_dependencies()
+        except ResourceProgramError:
+            resource_deps = ()
+        result = set(itertools.chain(
+            zip(itertools.repeat(direct), direct_deps),
+            zip(itertools.repeat(resource), resource_deps)))
+        return result
 
     def get_inhibitor_list(self, session_state, job):
         """
@@ -235,6 +245,10 @@ class CheckBoxSessionStateController(ISessionStateController):
         Analyze a result of a CheckBox "resource" job and generate
         or replace resource records.
         """
+        self._parse_and_store_resource(session_state, job, result)
+        self._instantiate_templates(session_state, job, result)
+
+    def _parse_and_store_resource(self, session_state, job, result):
         # NOTE: https://bugs.launchpad.net/checkbox/+bug/1297928
         # If we are resuming from a session that had a resource job that
         # never ran, we will see an empty MemoryJobResult object.
@@ -254,6 +268,29 @@ class CheckBoxSessionStateController(ISessionStateController):
         # Replace any old resources with the new resource list
         session_state.set_resource_list(job.id, new_resource_list)
 
+    def _instantiate_templates(self, session_state, job, result):
+        # NOTE: https://bugs.launchpad.net/checkbox/+bug/1297928
+        # If we are resuming from a session that had a resource job that
+        # never ran, we will see an empty MemoryJobResult object.
+        # Processing empty I/O log would create an empty resource list
+        # and that state is different from the state the session started
+        # before it was suspended, so don't
+        if result.outcome is IJobResult.OUTCOME_NONE:
+            return
+        for unit in session_state.unit_list:
+            if isinstance(unit, TemplateUnit) and unit.resource_id == job.id:
+                logger.info(_("Instantiating unit: %s"), unit)
+                for new_unit in unit.instantiate_all(
+                        session_state.resource_map[job.id]):
+                    try:
+                        new_unit.validate()
+                    except ValidationError as exc:
+                        logger.error(
+                            _("Ignoring invalid instantiated unit %s: %s"),
+                            new_unit, exc)
+                    else:
+                        session_state.add_unit(new_unit)
+
     def _process_local_result(self, session_state, job, result):
         """
         Analyze a result of a CheckBox "local" job and generate
@@ -264,7 +301,13 @@ class CheckBoxSessionStateController(ISessionStateController):
         new_job_list = []
         for record in gen_rfc822_records_from_io_log(job, result):
             new_job = job.create_child_job_from_record(record)
-            new_job_list.append(new_job)
+            try:
+                new_job.validate()
+            except ValidationError as exc:
+                logger.error(_("Ignoring invalid generated job %s: %s"),
+                             new_job.id, exc)
+            else:
+                new_job_list.append(new_job)
         # Then for each new job, add it to the job_list, unless it collides
         # with another job with the same id.
         for new_job in new_job_list:
@@ -315,7 +358,7 @@ def gen_rfc822_records_from_io_log(job, result):
             # TRANSLATORS: keep the word "local" untranslated. It is a
             # special type of job that needs to be distinguished.
             _("local script %s returned invalid RFC822 data: %s"),
-            job, exc)
+            job.id, exc)
 
 
 checkbox_session_state_ctrl = CheckBoxSessionStateController()
@@ -359,23 +402,18 @@ class CheckBoxExecutionController(IExecutionController):
     controllers.
     """
 
-    def __init__(self, session_dir, provider_list):
+    def __init__(self, provider_list):
         """
         Initialize a new CheckBoxExecutionController
 
-        :param session_dir:
-            Base directory of the session this job will execute in.
-            This directory is used to co-locate some data that is unique to
-            this execution as well as data that is shared by all executions.
         :param provider_list:
             A list of Provider1 objects that will be available for script
             dependency resolutions. Currently all of the scripts are makedirs
             available but this will be refined to the minimal set later.
         """
-        self._session_dir = session_dir
         self._provider_list = provider_list
 
-    def execute_job(self, job, config, extcmd_popen):
+    def execute_job(self, job, config, session_dir, extcmd_popen):
         """
         Execute the specified job using the specified subprocess-like object
 
@@ -387,6 +425,10 @@ class CheckBoxExecutionController(IExecutionController):
             provide values for missing environment variables that are required
             by the job (as expressed by the environ key in the job definition
             file).
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param extcmd_popen:
             A subprocess.Popen like object
         :returns:
@@ -394,18 +436,21 @@ class CheckBoxExecutionController(IExecutionController):
         """
         # CHECKBOX_DATA is where jobs can share output.
         # It has to be an directory that scripts can assume exists.
-        if not os.path.isdir(self.CHECKBOX_DATA):
-            os.makedirs(self.CHECKBOX_DATA)
+        if not os.path.isdir(self.get_CHECKBOX_DATA(session_dir)):
+            os.makedirs(self.get_CHECKBOX_DATA(session_dir))
         # Setup the executable nest directory
         with self.configured_filesystem(job, config) as nest_dir:
             # Get the command and the environment.
             # of this execution controller
-            cmd = self.get_execution_command(job, config, nest_dir)
-            env = self.get_execution_environment(job, config, nest_dir)
-            # run the command
-            logger.debug(_("job[%s] executing %r with env %r"),
-                         job.id, cmd, env)
-            return extcmd_popen.call(cmd, env=env)
+            cmd = self.get_execution_command(
+                job, config, session_dir, nest_dir)
+            env = self.get_execution_environment(
+                job, config, session_dir, nest_dir)
+            with self.temporary_cwd(job, config) as cwd_dir:
+                # run the command
+                logger.debug(_("job[%s] executing %r with env %r in cwd %r"),
+                             job.id, cmd, env, cwd_dir)
+                return extcmd_popen.call(cmd, env=env, cwd=cwd_dir)
 
     @contextlib.contextmanager
     def configured_filesystem(self, job, config):
@@ -433,9 +478,79 @@ class CheckBoxExecutionController(IExecutionController):
             for provider in self._provider_list:
                 if job.provider.namespace == provider.namespace:
                     nest.add_provider(provider)
-            logger.debug(_("Symlink nest for executables: %s"), nest_dir)
             yield nest_dir
 
+    @contextlib.contextmanager
+    def temporary_cwd(self, job, config):
+        """
+        Context manager for handling temporary current working directory
+        for a particular execution of a job definition command.
+
+        :param job:
+            The JobDefinition to execute
+        :param config:
+            A PlainBoxConfig instance which can be used to load missing
+            environment definitions that apply to all jobs. It is used to
+            provide values for missing environment variables that are required
+            by the job (as expressed by the environ key in the job definition
+            file).
+        :returns:
+            Pathname of the new temporary directory
+        """
+        # Create a nest for all the private executables needed for execution
+        prefix = 'cwd-'
+        suffix = '.{}'.format(job.checksum)
+        with tempfile.TemporaryDirectory(suffix, prefix) as cwd_dir:
+            logger.debug(
+                _("Job temporary current working directory: %s"), cwd_dir)
+            try:
+                yield cwd_dir
+            finally:
+                leftovers = self._find_leftovers(cwd_dir)
+                if leftovers:
+                    self.on_leftover_files(job, config, cwd_dir, leftovers)
+
+    def _find_leftovers(self, cwd_dir):
+        """
+        Find left-over files and directories
+
+        :param cwd_dir:
+            Directory to inspect for leftover files
+        :returns:
+            A list of discovered files and directories (except for the cwd_dir
+            itself)
+        """
+        leftovers = []
+        for dirpath, dirnames, filenames in os.walk(cwd_dir):
+            if dirpath != cwd_dir:
+                leftovers.append(dirpath)
+            leftovers.extend(
+                os.path.join(dirpath, filename)
+                for filename in filenames)
+        return leftovers
+
+    @Signal.define
+    def on_leftover_files(self, job, config, cwd_dir, leftovers):
+        """
+        Handle any files left over by the execution of a job definition.
+
+        :param job:
+            job definition with the command and environment definitions
+        :param config:
+            configuration object (a PlainBoxConfig instance)
+        :param cwd_dir:
+            Temporary directory set as current working directory during job
+            definition command execution. During the time this signal is
+            emitted that directory still exists.
+        :param leftovers:
+            List of absolute pathnames of files and directories that were
+            created in the current working directory (cwd_dir).
+
+        .. note::
+            Anyone listening to this signal does not need to remove any of the
+            files. They are removed automatically after this method returns.
+        """
+
     def get_score(self, job):
         """
         Compute how applicable this controller is for the specified job.
@@ -464,7 +579,7 @@ class CheckBoxExecutionController(IExecutionController):
         """
 
     @abc.abstractmethod
-    def get_execution_command(self, job, config, nest_dir):
+    def get_execution_command(self, job, config, session_dir, nest_dir):
         """
         Get the command to execute the specified job
 
@@ -476,6 +591,10 @@ class CheckBoxExecutionController(IExecutionController):
             provide values for missing environment variables that are required
             by the job (as expressed by the environ key in the job definition
             file).
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. This argument may or may not be used,
@@ -485,7 +604,7 @@ class CheckBoxExecutionController(IExecutionController):
             List of command arguments
         """
 
-    def get_execution_environment(self, job, config, nest_dir):
+    def get_execution_environment(self, job, config, session_dir, nest_dir):
         """
         Get the environment required to execute the specified job:
 
@@ -497,6 +616,10 @@ class CheckBoxExecutionController(IExecutionController):
             provide values for missing environment variables that are required
             by the job (as expressed by the environ key in the job definition
             file).
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. This argument may or may not be used,
@@ -521,6 +644,14 @@ class CheckBoxExecutionController(IExecutionController):
             for name in list(env.keys()):
                 if name.startswith("LC_"):
                     del env[name]
+        else:
+            # Set the per-provider gettext domain and locale directory
+            if job.provider.gettext_domain is not None:
+                env['TEXTDOMAIN'] = env['PLAINBOX_PROVIDER_GETTEXT_DOMAIN'] = \
+                    job.provider.gettext_domain
+            if job.provider.locale_dir is not None:
+                env['TEXTDOMAINDIR'] = env['PLAINBOX_PROVIDER_LOCALE_DIR'] = \
+                    job.provider.locale_dir
         # Use PATH that can lookup checkbox scripts
         if job.provider.extra_PYTHONPATH:
             env['PYTHONPATH'] = os.pathsep.join(
@@ -531,8 +662,8 @@ class CheckBoxExecutionController(IExecutionController):
             [nest_dir]
             + env.get("PATH", "").split(os.pathsep))
         # Add per-session shared state directory
-        env['PLAINBOX_SESSION_SHARE'] = self.CHECKBOX_DATA
-        env['CHECKBOX_DATA'] = self.CHECKBOX_DATA
+        env['PLAINBOX_SESSION_SHARE'] = env['CHECKBOX_DATA'] = \
+            self.get_CHECKBOX_DATA(session_dir)
         # Add a path to the per-provider data directory
         if job.provider.data_dir is not None:
             env['PLAINBOX_PROVIDER_DATA'] = job.provider.data_dir
@@ -555,15 +686,15 @@ class CheckBoxExecutionController(IExecutionController):
                 env[env_var] = config.environment[env_var]
         return env
 
-    @property
-    def CHECKBOX_DATA(self):
+    def get_CHECKBOX_DATA(self, session_dir):
         """
         value of the CHECKBOX_DATA environment variable.
 
         This variable names a sub-directory of the session directory
         where jobs can share data between invocations.
         """
-        return os.path.join(self._session_dir, "CHECKBOX_DATA")
+        # TODO, rename this, it's about time now
+        return os.path.join(session_dir, "CHECKBOX_DATA")
 
     def get_warm_up_for_job(self, job):
         """
@@ -579,7 +710,7 @@ class UserJobExecutionController(CheckBoxExecutionController):
     An execution controller that works for jobs invoked as the current user.
     """
 
-    def get_execution_command(self, job, config, nest_dir):
+    def get_execution_command(self, job, config, session_dir, nest_dir):
         """
         Get the command to execute the specified job
 
@@ -588,6 +719,11 @@ class UserJobExecutionController(CheckBoxExecutionController):
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Ignored.
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
+            Ignored.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. Ingored.
@@ -603,7 +739,7 @@ class UserJobExecutionController(CheckBoxExecutionController):
         Compute how applicable this controller is for the specified job.
 
         :returns:
-            1 for jobs without a user override, 3 for jobs with user override
+            1 for jobs without a user override, 4 for jobs with user override
             if the invoking uid is 0 (root), -1 otherwise
         """
         if job.user is None:
@@ -624,7 +760,8 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
     difference between the target environment and the current environment.
     """
 
-    def get_differential_execution_environment(self, job, config, nest_dir):
+    def get_differential_execution_environment(
+            self, job, config, session_dir, nest_dir):
         """
         Get the environment required to execute the specified job:
 
@@ -636,6 +773,10 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
             provide values for missing environment variables that are required
             by the job (as expressed by the environ key in the job definition
             file).
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. This is simply passed to
@@ -651,7 +792,8 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
         are always retained.
         """
         base_env = os.environ
-        target_env = super().get_execution_environment(job, config, nest_dir)
+        target_env = super().get_execution_environment(
+            job, config, session_dir, nest_dir)
         return {
             key: value
             for key, value in target_env.items()
@@ -659,7 +801,7 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
             or key in job.get_environ_settings()
         }
 
-    def get_execution_environment(self, job, config, nest_dir):
+    def get_execution_environment(self, job, config, session_dir, nest_dir):
         """
         Get the environment required to execute the specified job:
 
@@ -669,6 +811,11 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Ignored.
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
+            Ignored.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. Ignored.
@@ -686,11 +833,11 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
     Execution controller that gains root using plainbox-trusted-launcher-1
     """
 
-    def __init__(self, session_dir, provider_list):
+    def __init__(self, provider_list):
         """
         Initialize a new RootViaPTL1ExecutionController
         """
-        super().__init__(session_dir, provider_list)
+        super().__init__(provider_list)
         # Ask pkaction(1) if the "run-plainbox-job" policykit action is
         # registered on this machine.
         action_id = b"org.freedesktop.policykit.pkexec.run-plainbox-job"
@@ -700,11 +847,16 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
         try:
             result = check_output(["pkaction", "--action-id", action_id],
                                   stderr=STDOUT)
+        except OSError as exc:
+            logger.warning(
+                _("Cannot check if plainbox-trusted-launcher-1 is"
+                  " available: %s"), str(exc))
+            result = b""
         except CalledProcessError as exc:
             result = exc.output
         self.is_supported = True if result.strip() == action_id else False
 
-    def get_execution_command(self, job, config, nest_dir):
+    def get_execution_command(self, job, config, session_dir, nest_dir):
         """
         Get the command to invoke.
 
@@ -714,6 +866,10 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Passed to
             :meth:`get_differential_execution_environment()`.
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. Passed to
@@ -733,13 +889,13 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
         if job.via is not None:
             cmd += ['--generator', job.via]
             parent_env = self.get_differential_execution_environment(
-                job.origin.source.job, config, nest_dir)
+                job.origin.source.job, config, session_dir, nest_dir)
             for key, value in sorted(parent_env.items()):
                 cmd += ['-G', '{}={}'.format(key, value)]
         # Run the specified target job in the specified environment
         cmd += ['--target', job.checksum]
         env = self.get_differential_execution_environment(
-            job, config, nest_dir)
+            job, config, session_dir, nest_dir)
         for key, value in sorted(env.items()):
             cmd += ['-T', '{}={}'.format(key, value)]
         return cmd
@@ -809,7 +965,7 @@ class RootViaPkexecExecutionController(
     root from the non-system-wide location.
     """
 
-    def get_execution_command(self, job, config, nest_dir):
+    def get_execution_command(self, job, config, session_dir, nest_dir):
         """
         Get the command to invoke.
 
@@ -819,6 +975,10 @@ class RootViaPkexecExecutionController(
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Passed to
             :meth:`get_differential_execution_environment()`.
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. Passed to
@@ -833,7 +993,7 @@ class RootViaPkexecExecutionController(
         cmd = ['pkexec', '--user', job.user, 'env']
         # Append all environment data
         env = self.get_differential_execution_environment(
-            job, config, nest_dir)
+            job, config, session_dir, nest_dir)
         cmd += ["{key}={value}".format(key=key, value=value)
                 for key, value in sorted(env.items())]
         # Lastly use job.shell -c, to run our command
@@ -871,16 +1031,11 @@ class RootViaSudoExecutionController(
     and over again.
     """
 
-    def __init__(self, session_dir, provider_list):
+    def __init__(self, provider_list):
         """
         Initialize a new RootViaSudoExecutionController
-
-        :param session_dir:
-            Base directory of the session this job will execute in.
-            This directory is used to co-locate some data that is unique to
-            this execution as well as data that is shared by all executions.
         """
-        super().__init__(session_dir, provider_list)
+        super().__init__(provider_list)
         # Check if the user can use 'sudo' on this machine. This check is a bit
         # Ubuntu specific and can be wrong due to local configuration but
         # without a better API all we can do is guess.
@@ -896,7 +1051,7 @@ class RootViaSudoExecutionController(
             in_admin_group = False
         self.user_can_sudo = in_sudo_group or in_admin_group
 
-    def get_execution_command(self, job, config, nest_dir):
+    def get_execution_command(self, job, config, session_dir, nest_dir):
         """
         Get the command to invoke.
 
@@ -905,6 +1060,10 @@ class RootViaSudoExecutionController(
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Ignored.
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
         :param nest_dir:
             A directory with a nest of symlinks to all executables required to
             execute the specified job. Ingored.
@@ -918,7 +1077,7 @@ class RootViaSudoExecutionController(
         cmd = ['sudo', '-u', job.user, 'env']
         # Append all environment data
         env = self.get_differential_execution_environment(
-            job, config, nest_dir)
+            job, config, session_dir, nest_dir)
         cmd += ["{key}={value}".format(key=key, value=value)
                 for key, value in sorted(env.items())]
         # Lastly use job.shell -c, to run our command
