@@ -270,13 +270,14 @@ class JobRunner(IJobRunner):
             execution controllers to be instantiated and used. In special cases
             it may be required to override this.
         """
+        self._session_dir = session_dir
         if execution_ctrl_list is None:
             execution_ctrl_list = [
-                RootViaPTL1ExecutionController(session_dir, provider_list),
-                RootViaPkexecExecutionController(session_dir, provider_list),
+                RootViaPTL1ExecutionController(provider_list),
+                RootViaPkexecExecutionController(provider_list),
                 # XXX: maybe this one should be only used on command line
-                RootViaSudoExecutionController(session_dir, provider_list),
-                UserJobExecutionController(session_dir, provider_list),
+                RootViaSudoExecutionController(provider_list),
+                UserJobExecutionController(provider_list),
             ]
         self._jobs_io_log_dir = jobs_io_log_dir
         # NOTE: deprecated
@@ -284,6 +285,24 @@ class JobRunner(IJobRunner):
         self._job_runner_ui_delegate = JobRunnerUIDelegate()
         self._dry_run = dry_run
         self._execution_ctrl_list = execution_ctrl_list
+        self._log_leftovers = True
+
+    @property
+    def log_leftovers(self):
+        """
+        flag controlling if leftover files should be logged
+
+        If you wish to connect a custom handler to :meth:`on_leftover_files()`
+        then it is advisable to set this property to False so that leftover
+        files are not handled twice
+
+        By default, this property is True and a detailed warning is logged
+        """
+        return self._log_leftovers
+
+    @log_leftovers.setter
+    def log_leftovers(self, value):
+        self._log_leftovers = value
 
     def get_warm_up_sequence(self, job_list):
         """
@@ -716,7 +735,7 @@ class JobRunner(IJobRunner):
         """
         # Bail early if there is nothing do do
         if job.command is None:
-            return None, ()
+            raise ValueError(_("job {0} has no command to run").format(job.id))
         # Get an extcmd delegate for observing all the IO the way we need
         delegate, io_log_gen = self._prepare_io_handling(job, config)
         # Create a subprocess.Popen() like object that uses the delegate
@@ -749,7 +768,12 @@ class JobRunner(IJobRunner):
 
     def _run_extcmd(self, job, config, extcmd_popen):
         ctrl = self._get_ctrl_for_job(job)
-        return ctrl.execute_job(job, config, extcmd_popen)
+        ctrl.on_leftover_files.connect(self.on_leftover_files)
+        try:
+            return ctrl.execute_job(
+                job, config, self._session_dir, extcmd_popen)
+        finally:
+            ctrl.on_leftover_files.disconnect(self.on_leftover_files)
 
     def _get_ctrl_for_job(self, job):
         """
@@ -779,3 +803,35 @@ class JobRunner(IJobRunner):
             _("Selected execution controller %s (score %d) for job %r"),
             ctrl.__class__.__name__, score, job.id)
         return ctrl
+
+    @Signal.define
+    def on_leftover_files(self, job, config, cwd_dir, leftovers):
+        """
+        Handle any files left over by the execution of a job definition.
+
+        :param job:
+            job definition with the command and environment definitions
+        :param config:
+            configuration object (a PlainBoxConfig instance)
+        :param cwd_dir:
+            Temporary directory set as current working directory during job
+            definition command execution. During the time this signal is
+            emitted that directory still exists.
+        :param leftovers:
+            List of absolute pathnames of files and directories that were
+            created in the current working directory (cwd_dir).
+
+        .. note::
+            Anyone listening to this signal does not need to remove any of the
+            files. They are removed automatically after this method returns.
+        """
+        if self._log_leftovers:
+            logger.warning(
+                _("Job {0} created leftover filesystem artefacts"
+                  " in its working directory").format(job.id))
+            for item in leftovers:
+                logger.warning(_("Leftover file/directory: %r"),
+                               os.path.relpath(item, cwd_dir))
+            logger.warning(
+                _("Please store desired files in $PLAINBOX_SESSION_SHARE and"
+                  " use regular temporary files for everything else"))
