@@ -29,9 +29,10 @@ from inspect import cleandoc
 from unittest import TestCase
 import os
 
-from plainbox.impl.job import JobDefinition, JobOutputTextSource
+from plainbox.impl.job import JobDefinition
 from plainbox.impl.secure.launcher1 import TrustedLauncher
 from plainbox.impl.secure.launcher1 import main
+from plainbox.impl.secure.origin import JobOutputTextSource
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.providers.v1 import Provider1PlugIn
 from plainbox.impl.secure.providers.v1 import all_providers
@@ -85,8 +86,9 @@ class TrustedLauncherTests(TestCase):
         env = {'key': 'value'}
         # Run the tested method
         retval = self.launcher.run_shell_from_job(job.checksum, env)
-        # Ensure that we run the job command via bash
-        mock_call.assert_called_once_with(['bash', '-c', job.command], env=env)
+        # Ensure that we run the job command via job.shell
+        mock_call.assert_called_once_with(
+            [job.shell, '-c', job.command], env=env)
         # Ensure that the return value of subprocess.call() is returned
         self.assertEqual(retval, mock_call())
 
@@ -100,11 +102,11 @@ class TrustedLauncherTests(TestCase):
         env = {'key': 'value'}
         # Run the tested method
         retval = self.launcher.run_shell_from_job(job.checksum, env)
-        # Ensure that we run the job command via bash with a preserved env
+        # Ensure that we run the job command via job.shell with a preserved env
         expected_env = dict(os.environ)
         expected_env.update(env)
-        mock_call.assert_called_once_with(['bash', '-c', job.command],
-                                          env=expected_env)
+        mock_call.assert_called_once_with(
+            [job.shell, '-c', job.command], env=expected_env)
         # Ensure that the return value of subprocess.call() is returned
         self.assertEqual(retval, mock_call())
 
@@ -124,9 +126,9 @@ class TrustedLauncherTests(TestCase):
         mock_load_rfc822_records.return_value = [record1, record2]
         # Run the tested method
         job_list = self.launcher.run_local_job(job.checksum, None)
-        # Ensure that we run the job command via bash
+        # Ensure that we run the job command via job.shell
         mock_check_output.assert_called_with(
-            ['bash', '-c', job.command], env={}, universal_newlines=True)
+            [job.shell, '-c', job.command], env={}, universal_newlines=True)
         # Ensure that we parse all of the output
         mock_load_rfc822_records.assert_called_with(
             mock_check_output(), source=JobOutputTextSource(job))
@@ -169,6 +171,8 @@ class MainTests(TestCase):
                                            [-g CHECKSUM]
                                            [-G NAME=VALUE [NAME=VALUE ...]]
 
+        Security elevation mechanism for plainbox
+
         optional arguments:
           -h, --help            show this help message and exit
           -w, --warmup          return immediately, only useful when used with
@@ -316,7 +320,7 @@ class MainTests(TestCase):
         verify what happens when `plainbox-trusted-launcher-1` is called with a
         checksum that cannot be found in any of the providers.
         """
-       # Run the program with io intercept
+        # Run the program with io intercept
         with TestIO(combined=True) as io:
             with self.assertRaises(SystemExit) as call:
                 main(['--target=1234', '-T', 'blarg'])
