@@ -5,9 +5,9 @@
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU General Public License version 3,
+# as published by the Free Software Foundation.
+
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -57,9 +57,9 @@ class DependencyCycleErrorTests(TestCase):
 
     def test_repr(self):
         expected = ("<DependencyCycleError job_list:["
-                    "<JobDefinition name:'A' plugin:'dummy'>, "
-                    "<JobDefinition name:'B' plugin:'dummy'>, "
-                    "<JobDefinition name:'A' plugin:'dummy'>]>")
+                    "<JobDefinition id:'A' plugin:'dummy'>, "
+                    "<JobDefinition id:'B' plugin:'dummy'>, "
+                    "<JobDefinition id:'A' plugin:'dummy'>]>")
         observed = repr(self.exc)
         self.assertEqual(expected, observed)
 
@@ -85,9 +85,9 @@ class DependencyMissingErrorTests(TestCase):
         self.assertIs(self.exc_direct.affecting_job, None)
         self.assertIs(self.exc_resource.affecting_job, None)
 
-    def test_missing_job_name(self):
-        self.assertEqual(self.exc_direct.missing_job_name, 'B')
-        self.assertEqual(self.exc_resource.missing_job_name, 'B')
+    def test_missing_job_id(self):
+        self.assertEqual(self.exc_direct.missing_job_id, 'B')
+        self.assertEqual(self.exc_resource.missing_job_id, 'B')
 
     def test_str_direct(self):
         expected = "missing dependency: 'B' (direct)"
@@ -101,16 +101,16 @@ class DependencyMissingErrorTests(TestCase):
 
     def test_repr_direct(self):
         expected = ("<DependencyMissingError "
-                    "job:<JobDefinition name:'A' plugin:'dummy'> "
-                    "missing_job_name:'B' "
+                    "job:<JobDefinition id:'A' plugin:'dummy'> "
+                    "missing_job_id:'B' "
                     "dep_type:'direct'>")
         observed = repr(self.exc_direct)
         self.assertEqual(expected, observed)
 
     def test_repr_resource(self):
         expected = ("<DependencyMissingError "
-                    "job:<JobDefinition name:'A' plugin:'dummy'> "
-                    "missing_job_name:'B' "
+                    "job:<JobDefinition id:'A' plugin:'dummy'> "
+                    "missing_job_id:'B' "
                     "dep_type:'resource'>")
         observed = repr(self.exc_resource)
         self.assertEqual(expected, observed)
@@ -136,38 +136,20 @@ class DependencyDuplicateErrorTests(TestCase):
         self.assertIs(self.exc.affecting_job, self.another_A)
 
     def test_str(self):
-        expected = "duplicate job name: 'A'"
+        expected = "duplicate job id: 'A'"
         observed = str(self.exc)
         self.assertEqual(expected, observed)
 
     def test_repr(self):
         expected = ("<DependencyDuplicateError "
-                    "job:<JobDefinition name:'A' plugin:'dummy'> "
-                    "duplicate_job:<JobDefinition name:'A' plugin:'dummy'>>")
+                    "job:<JobDefinition id:'A' plugin:'dummy'> "
+                    "duplicate_job:<JobDefinition id:'A' plugin:'dummy'>>")
         observed = repr(self.exc)
         self.assertEqual(expected, observed)
 
 
 class DependencySolverInternalsTests(TestCase):
 
-    def test_get_dependency_set_empty(self):
-        A = make_job('A')
-        expected = set()
-        observed = DependencySolver._get_dependency_set(A)
-        self.assertEqual(expected, observed)
-
-    def test_get_dependency_set_direct_one(self):
-        A = make_job('A', depends='B')
-        expected = set([("direct", 'B')])
-        observed = DependencySolver._get_dependency_set(A)
-        self.assertEqual(expected, observed)
-
-    def test_get_dependency_set_direct_two(self):
-        A = make_job('A', depends='B, C')
-        expected = set([("direct", 'B'), ("direct", 'C')])
-        observed = DependencySolver._get_dependency_set(A)
-        self.assertEqual(expected, observed)
-
     def test_get_job_map_produces_map(self):
         A = make_job('A')
         B = make_job('B')
@@ -194,9 +176,9 @@ class TestDependencySolver(TestCase):
     def test_direct_deps(self):
         # This tests the following simple job chain
         # A -> B -> C
-        A = make_job(name='A', depends='B')
-        B = make_job(name='B', depends='C')
-        C = make_job(name='C')
+        A = make_job(id='A', depends='B')
+        B = make_job(id='B', depends='C')
+        C = make_job(id='C')
         job_list = [A, B, C]
         expected = [C, B, A]
         observed = DependencySolver.resolve_dependencies(job_list)
@@ -206,10 +188,10 @@ class TestDependencySolver(TestCase):
         # This tests two independent job chains
         # A1 -> B1
         # A2 -> B2
-        A1 = make_job(name='A1', depends='B1')
-        B1 = make_job(name='B1',)
-        A2 = make_job(name='A2', depends='B2')
-        B2 = make_job(name='B2')
+        A1 = make_job(id='A1', depends='B1')
+        B1 = make_job(id='B1',)
+        A2 = make_job(id='A2', depends='B2')
+        B2 = make_job(id='B2')
         job_list = [A1, B1, A2, B2]
         expected = [B1, A1, B2, A2]
         observed = DependencySolver.resolve_dependencies(job_list)
@@ -220,8 +202,8 @@ class TestDependencySolver(TestCase):
         # A
         # B -> A
         # A will be visited twice
-        A = make_job(name='A')
-        B = make_job(name='B', depends='A')
+        A = make_job(id='A')
+        B = make_job(id='B', depends='A')
         job_list = [A, B]
         expected = [A, B]
         observed = DependencySolver.resolve_dependencies(job_list)
@@ -230,8 +212,8 @@ class TestDependencySolver(TestCase):
     def test_resource_deps(self):
         # This tests resource deps
         # A ~> R
-        A = make_job(name='A', requires='R.foo == "bar"')
-        R = make_job(name='R', plugin='resource')
+        A = make_job(id='A', requires='R.foo == "bar"')
+        R = make_job(id='R', plugin='resource')
         job_list = [A, R]
         expected = [R, A]
         observed = DependencySolver.resolve_dependencies(job_list)
@@ -246,35 +228,34 @@ class TestDependencySolver(TestCase):
         self.assertIs(call.exception.job, A)
         self.assertIs(call.exception.duplicate_job, another_A)
 
-
     def test_missing_direct_dependency(self):
         # This tests missing dependencies
         # A -> (inexisting B)
-        A = make_job(name='A', depends='B')
+        A = make_job(id='A', depends='B')
         job_list = [A]
         with self.assertRaises(DependencyMissingError) as call:
             DependencySolver.resolve_dependencies(job_list)
         self.assertIs(call.exception.job, A)
-        self.assertEqual(call.exception.missing_job_name, 'B')
+        self.assertEqual(call.exception.missing_job_id, 'B')
         self.assertEqual(call.exception.dep_type,
                          call.exception.DEP_TYPE_DIRECT)
 
     def test_missing_resource_dependency(self):
         # This tests missing resource dependencies
         # A ~> (inexisting R)
-        A = make_job(name='A', requires='R.attr == "value"')
+        A = make_job(id='A', requires='R.attr == "value"')
         job_list = [A]
         with self.assertRaises(DependencyMissingError) as call:
             DependencySolver.resolve_dependencies(job_list)
         self.assertIs(call.exception.job, A)
-        self.assertEqual(call.exception.missing_job_name, 'R')
+        self.assertEqual(call.exception.missing_job_id, 'R')
         self.assertEqual(call.exception.dep_type,
                          call.exception.DEP_TYPE_RESOURCE)
 
     def test_dependency_cycle_self(self):
         # This tests dependency loops
         # A -> A
-        A = make_job(name='A', depends='A')
+        A = make_job(id='A', depends='A')
         job_list = [A]
         with self.assertRaises(DependencyCycleError) as call:
             DependencySolver.resolve_dependencies(job_list)
@@ -283,8 +264,8 @@ class TestDependencySolver(TestCase):
     def test_dependency_cycle_simple(self):
         # This tests dependency loops
         # A -> B -> A
-        A = make_job(name='A', depends='B')
-        B = make_job(name='B', depends='A')
+        A = make_job(id='A', depends='B')
+        B = make_job(id='B', depends='A')
         job_list = [A, B]
         with self.assertRaises(DependencyCycleError) as call:
             DependencySolver.resolve_dependencies(job_list)
@@ -293,10 +274,10 @@ class TestDependencySolver(TestCase):
     def test_dependency_cycle_longer(self):
         # This tests dependency loops
         # A -> B -> C -> D -> B
-        A = make_job(name='A', depends='B')
-        B = make_job(name='B', depends='C')
-        C = make_job(name='C', depends='D')
-        D = make_job(name='D', depends='B')
+        A = make_job(id='A', depends='B')
+        B = make_job(id='B', depends='C')
+        C = make_job(id='C', depends='D')
+        D = make_job(id='D', depends='B')
         job_list = [A, B, C, D]
         with self.assertRaises(DependencyCycleError) as call:
             DependencySolver.resolve_dependencies(job_list)
@@ -305,8 +286,8 @@ class TestDependencySolver(TestCase):
     def test_dependency_cycle_via_resource(self):
         # This tests dependency loops
         # A -> R -> A
-        A = make_job(name='A', requires='R.key == "value"')
-        R = make_job(name='R', depends='A', plugin="resource")
+        A = make_job(id='A', requires='R.key == "value"')
+        R = make_job(id='R', depends='A', plugin="resource")
         job_list = [A, R]
         with self.assertRaises(DependencyCycleError) as call:
             DependencySolver.resolve_dependencies(job_list)
