@@ -45,8 +45,9 @@ from plainbox.impl.buildsystems import all_buildsystems
 from plainbox.impl.commands import ToolBase, CommandBase
 from plainbox.impl.job import JobDefinition
 from plainbox.impl.logging import setup_logging
-from plainbox.impl.providers.v1 import get_user_PROVIDERPATH_entry
+from plainbox.impl.providers.special import get_categories
 from plainbox.impl.providers.v1 import InsecureProvider1PlugInCollection
+from plainbox.impl.providers.v1 import get_user_PROVIDERPATH_entry
 from plainbox.impl.secure.config import Unset
 from plainbox.impl.secure.config import ValidationError \
     as ConfigValidationError
@@ -1029,6 +1030,10 @@ class ValidateCommand(ManageCommand):
         provider_list = all_providers.get_all_plugin_objects()
         if all(p.name != provider.name for p in provider_list):
             provider_list.append(provider)
+        # Add the built-in 'categories' provider, unless that's the one we're testing
+        categories_provider = get_categories()
+        if provider.base_dir != categories_provider.base_dir:
+            provider_list.append(categories_provider)
         _logger.info(_("Validating everything..."))
         unit_list, exc_list = self.collect_all_units(provider)
         early_issue_gen = self.get_early_issues(exc_list)
@@ -1046,7 +1051,8 @@ class ValidateCommand(ManageCommand):
             if ns.deprecated and issue.kind is Problem.deprecated:
                 show = True
             if ns.strict:
-                failed = True
+                if issue.severity is not Severity.advice:
+                    failed = True
                 show = True
             if show:
                 print(issue.relative_to(self.definition.location))
